@@ -19,6 +19,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <unistd.h>
 
 #include "aris-proof.h"
 #include "sen-parent.h"
@@ -35,40 +36,28 @@
 #include "list.h"
 #include "process.h"
 #include "sexpr-process.h"
+#include "conf-file.h"
+#include "aio.h"
 
 #define SUBPROOFS_DISABLED 0
 
-/* Menu Data Arrays */
-
-enum MENU_ORDER {
-  FILE_MENU = 0,
-  EDIT_MENU,
-  PROOF_MENU,
-  RULES_MENU,
-  FONT_MENU,
-  HELP_MENU,
-  NUM_MENUS
-};
-
-enum MENU_SIZE {
-  FILE_MENU_SIZE = 8,
-  EDIT_MENU_SIZE = 8,
-  PROOF_MENU_SIZE = 5,
-  RULES_MENU_SIZE = 2,
-  FONT_MENU_SIZE = 4,
-  HELP_MENU_SIZE = 2
-};
-
 // Main menu array.
 
+/*
 static mid_t head_text[NUM_MENUS] =
   {
-    {N_("File"), NULL, NULL, -1, -1, PARENT, -1},
-    {N_("Edit"), NULL, NULL, -1, -1, PARENT, -1},
-    {N_("Proof"), NULL, NULL, -1, -1, PARENT, -1},
-    {N_("Rules"), NULL, NULL, -1, -1, PARENT, -1},
-    {N_("Font"), NULL, NULL, -1, -1, PARENT, -1},
-    {N_("Help"), NULL, NULL, -1, -1, PARENT, -1}
+    {N_("File"), NULL, NULL, PARENT, -1},
+    {N_("Edit"), NULL, NULL, PARENT, -1},
+    {N_("Proof"), NULL, NULL, PARENT, -1},
+    {N_("Rules"), NULL, NULL, PARENT, -1},
+    {N_("Font"), NULL, NULL, PARENT, -1},
+    {N_("Help"), NULL, NULL, PARENT, -1}
+  };
+*/
+static const char * head_text[] =
+  {
+    N_("File"), N_("Edit"), N_("Proof"),
+    N_("Rules"), N_("Font"), N_("Help")
   };
 
 // Number of submenus for each menu.
@@ -76,75 +65,6 @@ static mid_t head_text[NUM_MENUS] =
 static int num_subs[NUM_MENUS] = { FILE_MENU_SIZE, EDIT_MENU_SIZE, PROOF_MENU_SIZE,
 				   RULES_MENU_SIZE, FONT_MENU_SIZE, HELP_MENU_SIZE };
 
-// The file menu array.
-
-static mid_t file_text[FILE_MENU_SIZE] =
-  {
-    {N_("New"), N_("Begin a new proof."), GTK_STOCK_NEW, GDK_KEY_n, GDK_CONTROL_MASK, STANDARD, MENU_NEW},
-    {N_("Open"), N_("Open a proof."), GTK_STOCK_OPEN, GDK_KEY_o, GDK_CONTROL_MASK, STANDARD, MENU_OPEN},
-    {NULL, NULL, NULL, -1, -1, SEPARATOR, -1},
-    {N_("Save"), N_("Save the current proof."), GTK_STOCK_SAVE, GDK_KEY_s, GDK_CONTROL_MASK, STANDARD, MENU_SAVE},
-    {N_("Save As"), N_("Save this proof under a different name."), GTK_STOCK_SAVE_AS, GDK_KEY_s, GDK_CONTROL_MASK | GDK_SHIFT_MASK, STANDARD, MENU_SAVE_AS },
-    {NULL, NULL, NULL, -1, -1, SEPARATOR, -1},
-    {N_("Close"), N_("Close the current proof."), GTK_STOCK_CLOSE, GDK_KEY_w, GDK_CONTROL_MASK, STANDARD, MENU_CLOSE},
-    {N_("Quit"), N_("Quit GNU Aris."), GTK_STOCK_QUIT, GDK_KEY_q, GDK_CONTROL_MASK, STANDARD, MENU_QUIT}
-  };
-
-// The edit menu array.
-
-static mid_t edit_text[EDIT_MENU_SIZE] =
-  {
-    {N_("Add Premise"), N_("Add a new premise to the current proof."), GTK_STOCK_YES, GDK_KEY_p, GDK_CONTROL_MASK, STANDARD, MENU_ADD_PREM},
-    {N_("Add Conclusion"), N_("Add a new conclusion to the current proof."), GTK_STOCK_NO, GDK_KEY_j, GDK_CONTROL_MASK, STANDARD, MENU_ADD_CONC},
-    {N_("Add Subproof"), N_("Add a new subproof to the current proof."), GTK_STOCK_MEDIA_NEXT, GDK_KEY_b, GDK_CONTROL_MASK, STANDARD, MENU_ADD_SUB},
-    {N_("End Subproof"), N_("End the current subproof."), GTK_STOCK_MEDIA_PREVIOUS, GDK_KEY_d, GDK_CONTROL_MASK, STANDARD, MENU_END_SUB},
-    {NULL, NULL, NULL, -1, -1, SEPARATOR, -1},
-    {N_("Copy Line"), N_("Copy the current line in the current proof."), GTK_STOCK_COPY, GDK_KEY_g, GDK_CONTROL_MASK, STANDARD, MENU_COPY},
-    {N_("Kill Line"), N_("Kill (Cut) the current line in the current proof."), GTK_STOCK_CUT, GDK_KEY_k, GDK_CONTROL_MASK, STANDARD, MENU_KILL},
-    {N_("Insert Line"), N_("Insert a copied/killed line after the current line in the current proof."), GTK_STOCK_PASTE, GDK_KEY_i, GDK_CONTROL_MASK, STANDARD, MENU_INSERT}
-  };
-
-// The proof menu array.
-
-static mid_t proof_text[PROOF_MENU_SIZE] =
-  {
-    {N_("Evaluate Line"), N_("Evaluate the current line in the current proof."), GTK_STOCK_EXECUTE, GDK_KEY_e, GDK_CONTROL_MASK, STANDARD, MENU_EVAL_LINE},
-    {N_("Evaluate Proof"), N_("Evaluate the current proof."), GTK_STOCK_SELECT_ALL, GDK_KEY_f, GDK_CONTROL_MASK, STANDARD, MENU_EVAL_PROOF},
-    {NULL, NULL, NULL, -1, -1, SEPARATOR, -1},
-    {N_("Goal..."), N_("Check/Modify the current goal(s) for the current proof."), GTK_STOCK_INDEX, GDK_KEY_l, GDK_CONTROL_MASK,  STANDARD, MENU_GOAL},
-    {N_("Toggle Boolean Mode"), N_("Toggle Boolean mode for the current proof."), GTK_STOCK_CONVERT, GDK_KEY_m, GDK_CONTROL_MASK, STANDARD, MENU_BOOLEAN}
-  };
-
-// The rules menu array.
-
-static mid_t rules_text[RULES_MENU_SIZE] =
-  {
-    {N_("Toggle Rules"), N_("Show/Hide the rules tablet."), GTK_STOCK_REFRESH, GDK_KEY_r, GDK_CONTROL_MASK, STANDARD, MENU_TOGGLE_RULES},
-    {NULL, NULL, NULL, -1, -1, SEPARATOR, -1}
-  };
-
-// The font menu array.
-
-static mid_t font_text[FONT_MENU_SIZE] =
-  {
-    {N_("Small"), N_("Set the font size to small."), GTK_STOCK_ZOOM_OUT, GDK_KEY_minus, GDK_CONTROL_MASK, RADIO, MENU_SMALL},
-    {N_("Medium"), N_("Set the font size to medium."), GTK_STOCK_ZOOM_100, GDK_KEY_0, GDK_CONTROL_MASK, RADIO, MENU_MEDIUM},
-    {N_("Large"), N_("Set the font size to large."), GTK_STOCK_ZOOM_IN, GDK_KEY_equal, GDK_CONTROL_MASK, RADIO, MENU_LARGE},
-    {N_("Custom..."), N_("Set the font size manually."), GTK_STOCK_ZOOM_FIT, -1, 0, RADIO, MENU_CUSTOM}
-  };
-
-// The help menu array.
-
-static mid_t help_text[HELP_MENU_SIZE] =
-  {
-    {N_("Contents"), N_("Display help for GNU Aris."), GTK_STOCK_HELP, GDK_KEY_F1, 0, STANDARD, MENU_CONTENTS},
-    {N_("About GNU Aris"), N_("Display information about GNU Aris."), GTK_STOCK_ABOUT, -1, 0, STANDARD, MENU_ABOUT}
-  };
-
-// The array of arrays.
-
-static mid_t * menu_text[NUM_MENUS] = { file_text, edit_text, proof_text, rules_text, font_text, help_text };
-
 /* Initializes an aris proof.
  *  input:
  *    none.
@@ -164,7 +84,6 @@ aris_proof_init ()
 
   sen_parent_init (SEN_PARENT (ap), _("GNU Aris - Untitled"),
 		   640, 320, aris_proof_create_menu, SEN_PARENT_TYPE_PROOF);
-
   ap->boolean = the_app->boolean;
 
   if (the_app->boolean || SUBPROOFS_DISABLED)
@@ -207,7 +126,29 @@ aris_proof_init ()
   if (!ap->goal)
     return NULL;
 
-  aris_proof_set_font (ap, FONT_TYPE_SMALL);
+  gtk_accel_map_add_entry ("<ARIS-WINDOW>/Contents", GDK_KEY_F1, 0);
+
+  if (the_app->fonts[FONT_TYPE_CUSTOM])
+    {
+      aris_proof_set_font (ap, FONT_TYPE_CUSTOM);
+
+      // Set the small font size menu item to be sensitive.
+
+      GList * gl;
+      GtkWidget * font_menu, * font_submenu;
+
+      gl = gtk_container_get_children (GTK_CONTAINER (SEN_PARENT (ap)->menubar));
+      font_menu = (GtkWidget *) g_list_nth_data (gl, FONT_MENU);
+      font_submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (font_menu));
+
+      gl = gtk_container_get_children (GTK_CONTAINER (font_submenu));
+
+      gtk_widget_set_sensitive ((GtkWidget *) gl->data, TRUE);
+    }
+  else
+    {
+      aris_proof_set_font (ap, FONT_TYPE_SMALL);
+    }
 
   g_signal_connect (G_OBJECT (SEN_PARENT (ap)->window), "delete-event",
 		    G_CALLBACK (window_delete), (gpointer) ap);
@@ -215,6 +156,7 @@ aris_proof_init ()
 		    G_CALLBACK (window_focus_in), (gpointer) ap);
 
   gtk_widget_show_all (SEN_PARENT (ap)->window);
+
   return ap;
 }
 
@@ -343,7 +285,7 @@ aris_proof_init_from_proof (proof_t * proof)
 	  if (!sen)
 	    return NULL;
 
-	  sentence_set_bg (sen, BG_COLOR_WHITE);
+	  sentence_set_bg (sen, BG_COLOR_DEFAULT);
 	}
     }
 
@@ -423,21 +365,92 @@ aris_proof_create_menu (sen_parent * ap)
   int i, j;
   int got_radio = 0;
 
+  // I'm sure that there is an easier way to do this.
+
+  i = 0;
+
+  conf_obj main_file_array[] = {
+    main_menu_conf[CONF_MENU_NEW],
+    main_menu_conf[CONF_MENU_OPEN],
+    menu_separator,
+    main_menu_conf[CONF_MENU_SAVE],
+    main_menu_conf[CONF_MENU_SAVE_AS],
+    menu_separator,
+    main_menu_conf[CONF_MENU_CLOSE],
+    main_menu_conf[CONF_MENU_QUIT]
+  };
+
+  memcpy (main_file_menu, main_file_array,
+	  FILE_MENU_SIZE * sizeof (conf_obj));
+
+  conf_obj main_edit_array[] = {
+    main_menu_conf[CONF_MENU_ADD_PREM],
+    main_menu_conf[CONF_MENU_ADD_CONC],
+    main_menu_conf[CONF_MENU_ADD_SUB],
+    main_menu_conf[CONF_MENU_END_SUB],
+    menu_separator,
+    main_menu_conf[CONF_MENU_COPY],
+    main_menu_conf[CONF_MENU_KILL],
+    main_menu_conf[CONF_MENU_INSERT]
+  };
+
+  memcpy (main_edit_menu, main_edit_array,
+	  EDIT_MENU_SIZE * sizeof (conf_obj));
+
+  conf_obj main_proof_array[] = {
+    main_menu_conf[CONF_MENU_EVAL_LINE],
+    main_menu_conf[CONF_MENU_EVAL_PROOF],
+    menu_separator,
+    main_menu_conf[CONF_MENU_GOAL],
+    main_menu_conf[CONF_MENU_BOOLEAN]
+  };
+
+  memcpy (main_proof_menu, main_proof_array,
+	  PROOF_MENU_SIZE * sizeof (conf_obj));
+
+  conf_obj main_rule_array[] = {
+    main_menu_conf[CONF_MENU_TOGGLE_RULES],
+    menu_separator
+  };
+
+  memcpy (main_rules_menu, main_rule_array,
+	  RULES_MENU_SIZE * sizeof (conf_obj));
+
+  conf_obj main_font_array[] = {
+    main_menu_conf[CONF_MENU_SMALL],
+    main_menu_conf[CONF_MENU_MEDIUM],
+    main_menu_conf[CONF_MENU_LARGE],
+    main_menu_conf[CONF_MENU_CUSTOM],
+  };
+
+  memcpy (main_font_menu, main_font_array,
+	  FONT_MENU_SIZE * sizeof (conf_obj));
+
+  conf_obj main_help_array[] = {
+    main_menu_conf[CONF_MENU_CONTENTS],
+    main_menu_conf[CONF_MENU_ABOUT],
+  };
+
+  memcpy (main_help_menu, main_help_array,
+	  HELP_MENU_SIZE * sizeof (conf_obj));
+
   ap->menubar = gtk_menu_bar_new ();
 
-  for (i = 0; i < 6; i++)
+  for (i = 0; i < NUM_MENUS; i++)
     {
       submenu = gtk_menu_new ();
-      menu = gtk_menu_item_new_with_label (head_text[i].label);
-      mid_t * cur_data = menu_text[i];
+      menu = gtk_menu_item_new_with_label (head_text[i]);
+      gtk_menu_set_accel_group (GTK_MENU (submenu), ap->accel);
+      conf_obj * cur_data = main_menus_menu[i];
       for (j = 0; j < num_subs[i]; j++)
 	{
 	  construct_menu_item (cur_data[j], G_CALLBACK (menu_activate),
-			       submenu, ap->accel, &got_radio);
+			       submenu, &got_radio);
 	}
       gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
       gtk_menu_shell_append (GTK_MENU_SHELL (ap->menubar), menu);
     }
+
 }
 
 /* Sets an aris proof to be changed or not changed.
@@ -511,6 +524,51 @@ aris_proof_adjust_lines (aris_proof * ap, item_t * itm, int mod)
 	}
     }
 
+  // Find the corresponding menu item.
+  GtkWidget * menu_item, * menu, * submenu;
+  GList * gl;
+
+  gl = gtk_container_get_children (GTK_CONTAINER (ap->menubar));
+  menu = g_list_nth_data (gl, RULES_MENU);
+  submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
+
+  gl = gtk_container_get_children (GTK_CONTAINER (submenu));
+  for (; gl; gl = gl->next)
+    {
+      menu_item = gl->data;
+
+      if (GTK_IS_SEPARATOR_MENU_ITEM (menu_item))
+	continue;
+
+      const char * label =
+	gtk_menu_item_get_label (GTK_MENU_ITEM (gl->data));
+
+      int chk, line_num;
+      char * file_name;
+
+      file_name = (char *) calloc (strlen (label), sizeof (char));
+      CHECK_ALLOC (file_name, -1);
+
+      chk = sscanf (label, "%i - %s", &line_num, file_name);
+      if (chk != 2)
+	continue;
+
+      if (line_num < cur_line)
+	continue;
+
+      line_num += line_mod;
+
+      char * new_label;
+
+      new_label = (char *) calloc (strlen (label) + 1, sizeof (char));
+      CHECK_ALLOC (new_label, -1);
+
+      sprintf (new_label, "%i - %s", line_num, file_name);
+      free (file_name);
+
+      gtk_menu_item_set_label (GTK_MENU_ITEM (gl->data), new_label);
+    }
+
   return 0;
 }
 
@@ -773,10 +831,10 @@ aris_proof_set_font (aris_proof * ap, int font)
  *    none.
  */
 void
-aris_proof_set_sb (aris_proof * ap, char * sb_text)
+aris_proof_set_sb (aris_proof * ap, char * sbtext)
 {
-  sen_parent_set_sb (SEN_PARENT (ap), sb_text);
-  ap->sb_text = sb_text;
+  sen_parent_set_sb (SEN_PARENT (ap), sbtext);
+  ap->sb_text = sbtext;
 }
 
 /* Sets the file name of an aris proof.
@@ -902,7 +960,7 @@ aris_proof_kill (aris_proof * ap)
   item_t * sel_itr, * start_itr;
   int is_subproof = 0;
   int org_depth = -1;
- 
+
   if (ap->selected->num_stuff == 0)
     {
       sel_itr = SEN_PARENT (ap)->focused;
@@ -1040,3 +1098,148 @@ aris_proof_toggle_boolean_mode (aris_proof * ap)
 
   return 0;
 }
+
+int
+aris_proof_import_proof (aris_proof * ap)
+{
+  GtkFileFilter * file_filter;
+  file_filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (file_filter, "Aris Files");
+  gtk_file_filter_add_pattern (file_filter, "*.tle");
+
+  GtkWidget * file_chooser;
+  file_chooser =
+    gtk_file_chooser_dialog_new (_("Select a file to Open..."),
+				 GTK_WINDOW (ap->window),
+				 GTK_FILE_CHOOSER_ACTION_OPEN,
+				 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+				 NULL);
+  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (file_chooser), FALSE);
+  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (file_chooser), file_filter);
+
+  int ret;
+  ret = gtk_dialog_run (GTK_DIALOG (file_chooser));
+  gtk_widget_destroy (file_chooser);
+
+  if (ret != GTK_RESPONSE_ACCEPT)
+    return 0;
+
+  char * filename;
+  proof_t * proof;
+
+  filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
+
+  proof = aio_open (filename);
+  if (!proof)
+    {
+      gtk_widget_destroy (file_chooser);
+      return -1;
+    }
+
+  item_t * ev_itr, * pf_itr, * ev_conc = NULL;
+  int * refs = NULL, ref_num = 0;
+
+  refs = (int *) calloc (proof->everything->num_stuff, sizeof (int));
+  CHECK_ALLOC (refs, -1);
+  //TODO: allocate refs to the correct number of premises.
+
+  for (pf_itr = proof->everything->head; pf_itr;
+       pf_itr = pf_itr->next)
+    {
+      sen_data * sd;
+      char * pf_text;
+
+      sd = pf_itr->value;
+      if (!sd->premise)
+	break;
+
+      pf_text = sd->text;
+
+      for (ev_itr = ap->everything->head; ev_itr; ev_itr = ev_itr->next)
+	{
+	  char * ev_text;
+	  sentence * ev_sen;
+
+	  ev_sen = ev_itr->value;
+	  if (!ev_sen->premise)
+	    {
+	      if (!ev_conc)
+		ev_conc = ev_itr;
+	      break;
+	    }
+
+	  ev_text = ev_sen->text;
+
+	  if (!strcmp (ev_text, pf_text))
+	    {
+	      refs[ref_num++] = ev_sen->line_num;
+	      break;
+	    }
+	}
+
+      if (!ev_itr || !((sentence *) ev_itr->value)->premise)
+	{
+	  sentence * sen_chk;
+	  sen_chk = aris_proof_create_sentence (ap, sd);
+	  if (!sen_chk)
+	    return -1;
+	  refs[ref_num++] = sen_chk->line_num;
+	}
+    }
+
+  refs[ref_num] = -1;
+
+  if (!ev_conc)
+    {
+      for (ev_itr = ap->everything->head; ev_itr;
+	   ev_itr = ev_itr->next)
+	{
+	  sentence * ev_sen;
+
+	  ev_sen = ev_itr->value;
+	  if (!ev_sen->premise)
+	    {
+	      if (!ev_conc)
+		ev_conc = ev_itr;
+	      break;
+	    }
+	}
+    }
+
+  for (pf_itr = proof->goals->head; pf_itr; pf_itr = pf_itr->next)
+    {
+      unsigned char * pf_text;
+      pf_text = pf_itr->value;
+
+      for (ev_itr = ev_conc; ev_itr; ev_itr = ev_itr->next)
+	{
+	  sentence * ev_sen;
+	  unsigned char * ev_text;
+	  ev_sen = ev_itr->value;
+	  ev_text = ev_sen->text;
+
+	  if (!strcmp (pf_text, ev_text))
+	    break;
+	}
+
+      if (!ev_itr)
+	{
+	  sen_data * sd;
+	  sentence * sen_chk;
+
+	  sd = sen_data_init (-1, RULE_LM, pf_text, refs,
+			      0, filename, 0, -1, NULL);
+	  if (!sd)
+	    return -1;
+
+	  sen_chk = aris_proof_create_sentence (ap, sd);
+	  if (!sen_chk)
+	    return -1;
+	}
+    }
+
+  free (refs);
+  proof_destroy (proof);
+  return 0;
+}
