@@ -19,6 +19,8 @@
 #ifndef ARIS_INTEROP_ISAR_H
 #define ARIS_INTEROP_ISAR_H
 
+#include "typedef.h"
+
 struct input_type {
   char * type;
   char * seq;
@@ -29,9 +31,6 @@ struct key_function {
   int (* func) (char *, char **);
 };
 
-typedef struct input_type in_type;
-typedef struct key_function key_func;
-typedef struct proof proof_t;
 
 enum KEY_FUNCS {
   KF_SYN = 0,
