@@ -19,10 +19,7 @@
 #ifndef PROOF_H
 #define PROOF_H
 
-typedef struct proof proof_t;
-typedef struct list list_t;
-typedef struct item item_t;
-typedef struct vector vec_t;
+#include "typedef.h"
 
 // Proof data structure.
 
