@@ -106,13 +106,19 @@ recurse_co (unsigned char * sen_0, unsigned char * sen_1)
 	  return ret;
 	}
 
-      if ((!strncmp (sen_0 + 2, S_UNV, S_CL) || !strncmp (sen_0 + 2, S_EXL, S_CL))
-	  || (!strncmp (sen_1 + 2, S_UNV, S_CL) || !strncmp (sen_1 + 2, S_EXL, S_CL)))
+      if ((sen_0[0] == '(' && sen_1[0] == '(')
+	  && ((!strncmp (sen_0 + 2, S_UNV, S_CL)
+	       || !strncmp (sen_0 + 2, S_EXL, S_CL))
+	      || (!strncmp (sen_1 + 2, S_UNV, S_CL)
+		  || !strncmp (sen_1 + 2, S_EXL, S_CL))))
 	{
-	  if (!(strncmp (sen_0 + 2, S_UNV, S_CL) && strncmp (sen_0 + 2, S_EXL, S_CL))
+	  if (!(strncmp (sen_0 + 2, S_UNV, S_CL)
+		&& strncmp (sen_0 + 2, S_EXL, S_CL))
 	      && !(strncmp (sen_1 + 2, S_UNV, S_CL)
 		   && strncmp (sen_1 + 2, S_EXL, S_CL)))
-	    return -2;
+	    {
+	      return -2;
+	    }
 
 	  unsigned char * scope_0, * scope_1, * var_0, * var_1;
 	  unsigned char quant_0[S_CL + 1], quant_1[S_CL + 1];
@@ -280,8 +286,11 @@ recurse_id (unsigned char * sen_0, unsigned char * sen_1)
 	  return ret;
 	}
 
-      if ((!strncmp (sen_0 + 2, S_UNV, S_CL) || !strncmp (sen_0 + 2, S_EXL, S_CL))
-	  || (!strncmp (sen_1 + 2, S_UNV, S_CL) || !strncmp (sen_1 + 2, S_EXL, S_CL)))
+      if ((sen_0[0] == '(' && sen_1[0] == '(')
+	  && ((!strncmp (sen_0 + 2, S_UNV, S_CL)
+	       || !strncmp (sen_0 + 2, S_EXL, S_CL))
+	      || (!strncmp (sen_1 + 2, S_UNV, S_CL)
+		  || !strncmp (sen_1 + 2, S_EXL, S_CL))))
 	{
 	  if (!(strncmp (sen_0 + 2, S_UNV, S_CL) && strncmp (sen_0 + 2, S_EXL, S_CL))
 	      && !(strncmp (sen_1 + 2, S_UNV, S_CL)
@@ -578,12 +587,7 @@ proc_im (unsigned char * prem, unsigned char * conc)
   int oth_pos;
 
   oth_sen = (unsigned char * ) calloc (d_len - 2 - S_NL, sizeof (char));
-  if (!oth_sen)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
+  CHECK_ALLOC (oth_sen, NULL);
   strncpy (oth_sen, dis_sen, i - 1);
   oth_pos = i - 1;
 
@@ -627,7 +631,9 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
 	mode = 1;
     }
   else
-    mode = mode_guess;
+    {
+      mode = mode_guess;
+    }
 
   if (mode == 0)
     {
@@ -636,6 +642,9 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       unsigned char * not_sen, * oth_sen;
       int n_len;
 
+      if (i == 0)
+	return _("DeMorgan constructed incorrectly.");
+
       if (!strncmp (prem + i - 1, S_NOT, S_NL))
 	{
 	  not_sen = prem;  n_len = p_len;
@@ -706,16 +715,11 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       // Construct what should be the other sentence.
 
       unsigned char * cons_sen;
-      int cons_pos;
-
-      cons_sen = (unsigned char *) calloc (n_len + (gg_vec->num_stuff - 1) * S_NL + 1,
-					   sizeof (char));
-      if (!cons_sen)
-	{
-	  perror (NULL);
-	  return NULL;
-	}
+      int cons_pos, alloc_size;
 
+      alloc_size = n_len + (gg_vec->num_stuff - 1) * (S_NL + 3);
+      cons_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+      CHECK_ALLOC (cons_sen, NULL);
       strncpy (cons_sen, not_sen, i - 1);
       cons_pos = i - 1;
 
@@ -774,7 +778,7 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       if (!tmp_str)
 	return _("DeMorgan constructed incorrectly.");
 
-      elim_sen = elim_not (tmp_str);
+      elim_sen = sexpr_elim_not (tmp_str);
       if (!elim_sen)
 	return NULL;
       free (tmp_str);
@@ -782,39 +786,24 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       if (elim_sen[1] != '(')
 	return _("There must be a quantifier after the negation.");
 
-      int oth_pos;
-      oth_pos = parse_parens (elim_sen, 1, &tmp_str);
-      if (oth_pos == -2)
-	return NULL;
-
       unsigned char * cmp_str;
-      oth_pos = parse_parens (elim_sen, oth_pos + 2, &cmp_str);
-      if (oth_pos == -2)
+      int oth_pos;
+      unsigned char quant[S_CL + 1], oth_quant[S_CL + 1];
+      unsigned char * var;
+
+      cmp_str = sexpr_elim_quant (elim_sen, quant, &var);
+      if (!cmp_str)
 	return NULL;
 
-      unsigned char oth_quant[S_CL + 1];
+      if (cmp_str[0] == '\0')
+	return _("DeMorgan constructed incorrectly.");
 
-      if (!strncmp (tmp_str + 1, S_UNV, CL))
+      if (!strncmp (quant, S_UNV, S_CL))
 	strcpy (oth_quant, S_EXL);
-      else
+      else if (!strncmp (quant, S_EXL, S_CL))
 	strcpy (oth_quant, S_UNV);
-
-      oth_pos = S_CL + 2;
-      while (tmp_str[oth_pos] != ')')
-	oth_pos++;
-
-      unsigned char * var;
-      var = (unsigned char *) calloc (oth_pos + 1, sizeof (char));
-      if (!var)
-	{
-	  perror (NULL);
-	  return NULL;
-	}
-
-      strncpy (var, tmp_str + S_CL + 2, oth_pos);
-      var[oth_pos] = '\0';
-
-      free (tmp_str);
+      else
+	return _("There must be a quantifier after the negation.");
 
       // Construct what should be the other sentence.
 
@@ -822,20 +811,15 @@ proc_dm (unsigned char * prem, unsigned char * conc, int mode_guess)
       int cons_pos;
 
       cons_sen = (unsigned char *) calloc (n_len, sizeof (char));
-      if (!cons_sen)
-	{
-	  perror (NULL);
-	  return NULL;
-	}
-
-      strncpy (cons_sen, not_sen, i);
-      cons_pos = i;
+      CHECK_ALLOC (cons_sen, NULL);
+      strncpy (cons_sen, not_sen, i - 1);
+      cons_pos = i - 1;
       cons_pos += sprintf (cons_sen + cons_pos, "((%s %s) (%s %s))",
 			   oth_quant, var, S_NOT, cmp_str);
       free (cmp_str);
       free (var);
 
-      strcpy (cons_sen + cons_pos, not_sen + tmp_pos);
+      strcpy (cons_sen + cons_pos, not_sen + tmp_pos + 1);
 
       char * ret_str;
 
@@ -892,12 +876,19 @@ proc_as (unsigned char * prem, unsigned char * conc)
   if (sh_sen[si] == '(')
     si--;
 
+  if (si < 0)
+    return _("There must be a connective in both sentences.");
+
   si = find_unmatched_o_paren (sh_sen, si);
   if (si == -1)
     return _("There must be a connective in both sentences.");
 
   if (ln_sen[li] != '(')
-    li = find_unmatched_o_paren (ln_sen, li);
+    {
+      li = find_unmatched_o_paren (ln_sen, li);
+      if (li == -1)
+	return _("There must be a connective in both sentences.");
+    }
 
   int ret_chk;
   ret_chk = sexpr_find_unmatched (ln_sen, sh_sen, &li, &si);
@@ -913,6 +904,9 @@ proc_as (unsigned char * prem, unsigned char * conc)
 
   int ti;
   ti = li - 1;
+  if (ti < 0)
+    return _("Association constructed incorrectly.");
+
   ti = find_unmatched_o_paren (ln_sen, ti);
 
   if (strncmp (ln_sen + ti + 1, S_AND, S_CL) && strncmp (ln_sen + ti + 1, S_OR, S_CL)
@@ -925,17 +919,12 @@ proc_as (unsigned char * prem, unsigned char * conc)
   int oth_pos;
 
   oth_sen = (unsigned char *) calloc (l_len - 2 - S_CL, sizeof (char));
-  if (!oth_sen)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
+  CHECK_ALLOC (oth_sen, NULL);
   strncpy (oth_sen, ln_sen, li);
   oth_pos = li;
 
-  strncpy (oth_sen + oth_pos, ln_sen + li + S_CL + 2, tmp_pos - li - 3 - S_CL);
-  oth_pos += tmp_pos - li - 3 - S_CL;
+  strncpy (oth_sen + oth_pos, ln_sen + li + S_CL + 2, tmp_pos - li - 2 - S_CL);
+  oth_pos += tmp_pos - li - 2 - S_CL;
 
   strcpy (oth_sen + oth_pos, ln_sen + tmp_pos + 1);
 
@@ -1036,9 +1025,12 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
   if (i == -1)
     return NO_DIFFERENCE;
 
+  if (p_len == c_len)
+    return _("The two sentences must be of different lengths.");
+
   if (mode_guess == -1)
     {
-      if (ln_sen[i] == '(')
+      if (sh_sen[i] == '(')
 	mode = 1;
       else
 	mode = 0;
@@ -1052,6 +1044,14 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
     {
       int tmp_pos;
       unsigned char * tmp_str;
+
+      if (i < 2
+	  || (strncmp (sh_sen + i - 1, S_AND, S_CL)
+	      &&  strncmp (sh_sen + i - 1, S_OR, S_CL)))
+	{
+	  return _("There must be a connective at the difference.");
+	}
+
       tmp_pos = parse_parens (sh_sen, i - S_CL + 1, &tmp_str);
       if (tmp_pos == -2)
 	return NULL;
@@ -1114,12 +1114,7 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
       // r_len - 1 (both of the parentheses, plus one for the null byte).
       alloc_size = s_len + (strlen (lsen) + S_CL + 4) * rsens->num_stuff + r_len - 1;
       oth_sen = (unsigned char *) calloc (alloc_size,  sizeof (char));
-      if (!oth_sen)
-	{
-	  perror (NULL);
-	  return NULL;
-	}
-
+      CHECK_ALLOC (oth_sen, NULL);
       strncpy (oth_sen, sh_sen, i - 1);
       oth_pos = i - 1;
 
@@ -1157,6 +1152,12 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
       unsigned char * tmp_str;
       int tmp_pos;
 
+      if (mode_guess != -1)
+	{
+	  if (sh_sen[i] != '(')
+	    return _("The difference must be a quantifier.");
+	}
+
       tmp_pos = parse_parens (sh_sen, i - 1, &tmp_str);
       if (tmp_pos == -2)
 	return NULL;
@@ -1168,7 +1169,7 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
       if (!scope)
 	return NULL;
       if (scope[0] == '\0')
-	return _("ERROR");
+	return _("Distribution constructed incorrectly.");
       v_len = strlen (var);
       free (tmp_str);
 
@@ -1202,24 +1203,19 @@ proc_dt (unsigned char * prem, unsigned char * conc, int mode_guess)
       unsigned char * oth_sen;
       int oth_pos, j, alloc_size;
 
-      alloc_size = s_len + (gg_vec->num_stuff * (6 + S_CL + strlen (var))) + 1;
-      oth_sen = (unsigned char *) calloc (alloc_size, sizeof (char));
-      if (!oth_sen)
-	{
-	  perror (NULL);
-	  return NULL;
-	}
-
+      alloc_size = s_len + (gg_vec->num_stuff * (6 + S_CL + strlen (var)));
+      oth_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+      CHECK_ALLOC (oth_sen, NULL);
       strncpy (oth_sen, sh_sen, i - 1);
       oth_pos = i - 1;
 
-      oth_pos += sprintf (oth_sen + oth_pos, "(%s ", conn);
+      oth_pos += sprintf (oth_sen + oth_pos, "(%s", conn);
 
       for (j = 0; j < gg_vec->num_stuff; j++)
 	{
 	  unsigned char * cur_gen, * str;
 	  cur_gen = vec_str_nth (gg_vec, j);
-	  oth_pos += sprintf (oth_sen + oth_pos, "((%s %s) %s)",
+	  oth_pos += sprintf (oth_sen + oth_pos, " ((%s %s) %s)",
 			      quant, var, cur_gen);
 	}
 
@@ -1272,7 +1268,7 @@ proc_eq (unsigned char * prem, unsigned char * conc)
     return NO_DIFFERENCE;
 
   if (i < 2)
-    return _("Incorrect usage of equivalence.");
+    return _("Equivalence constructed incorrectly.");
 
   int tmp_pos;
   unsigned char * tmp_str;
@@ -1286,6 +1282,7 @@ proc_eq (unsigned char * prem, unsigned char * conc)
   int ftc;
   unsigned char * lsen, * rsen;
 
+  lsen = rsen = NULL;
   ftc = sexpr_find_top_connective (tmp_str, S_BIC, &lsen, &rsen);
   if (ftc == -1)
     return NULL;
@@ -1299,15 +1296,11 @@ proc_eq (unsigned char * prem, unsigned char * conc)
     }
 
   unsigned char * cons_sen;
-  int cons_pos;
-
-  cons_sen = (unsigned char *) calloc (b_len + (tmp_pos - (i - 2) + 1) + S_CL + 5, sizeof (char));
-  if (!cons_sen)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  int cons_pos, alloc_size;
 
+  alloc_size = b_len + (tmp_pos - (i - 2) + 1) + S_CL + 4;
+  cons_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+  CHECK_ALLOC (cons_sen, NULL);
   strncpy (cons_sen, bic_sen, i - 1);
   cons_pos = i - 1;
 
@@ -1360,15 +1353,15 @@ proc_dn (unsigned char * prem, unsigned char * conc)
   if (i == -1)
     return NO_DIFFERENCE;
 
-  if (ln_sen[i] != '(' && strncmp (ln_sen + i, NOT, NL)
-      && (i > 0 && strncmp (ln_sen + i - 1, NOT, NL)))
+  if (ln_sen[i] != '(' && strncmp (ln_sen + i, S_NOT, S_NL)
+      && (i > 0 && l_len > S_NL && strncmp (ln_sen + i - 1, S_NOT, S_NL)))
     {
       return _("The difference must be at a negation.");
     }
 
   if (ln_sen[i] != '(')
     {
-      if (!strncmp (ln_sen + i, NOT, NL))
+      if (!strncmp (ln_sen + i, S_NOT, S_NL))
 	i--;
       else
 	i -= 2;
@@ -1387,7 +1380,7 @@ proc_dn (unsigned char * prem, unsigned char * conc)
   int oth_pos;
 
   pos = i;
-  if (!sexpr_not_check (tmp_str) || !sexpr_not_check (tmp_str + 2 + S_NL))
+  if (!sexpr_not_check (tmp_str) || strncmp (tmp_str + S_NL + 3, S_NOT, S_NL))
     {
       free (tmp_str);
       return _("There must be at least two negations at the difference.");
@@ -1398,14 +1391,9 @@ proc_dn (unsigned char * prem, unsigned char * conc)
   // Begin removing pairs of negations, until there are no more pairs.
 
   oth_sen = (unsigned char *) calloc (l_len - S_NL * 2 - 5, sizeof (char));
-  if (!oth_sen)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
-  strncpy (oth_sen, ln_sen, i - 1);
-  oth_pos = i - 1;
+  CHECK_ALLOC (oth_sen, NULL);
+  strncpy (oth_sen, ln_sen, i);
+  oth_pos = i;
 
   strncpy (oth_sen + oth_pos, ln_sen + pos, tmp_pos - pos - 1);
   oth_pos += tmp_pos - pos - 1;
@@ -1438,9 +1426,6 @@ proc_ep (unsigned char * prem, unsigned char * conc)
   p_len = strlen (prem);
   c_len = strlen (conc);
 
-  if (p_len != c_len)
-    return _("The reference and conclusion should be of equal length.");
-
   // Count number of conditionals in each sentence.
 
   int i, c_cons, p_cons;
@@ -1456,17 +1441,15 @@ proc_ep (unsigned char * prem, unsigned char * conc)
 
   if (c_cons > p_cons)
     {
+      and_sen = prem;  a_len = p_len;
       con_sen = conc;
-      and_sen = prem;
     }
   else
     {
+      and_sen = conc;  a_len = c_len;
       con_sen = prem;
-      and_sen = conc;
     }
 
-  a_len = p_len;
-
   i = find_difference (and_sen, con_sen);
   if (i == -1)
     return NO_DIFFERENCE;
@@ -1476,14 +1459,22 @@ proc_ep (unsigned char * prem, unsigned char * conc)
   ai = ci = i;
 
   if (con_sen[ci] == '(')
-    ci--;
+    {
+      ci--;
+      if (ci == -1)
+	return _("Exportation constructed incorrectly.");
+    }
 
   ci = find_unmatched_o_paren (con_sen, ci);
   if (ci == -1)
     return _("There must be connectives on both sentences.");
 
   if (and_sen[ai] != '(')
-    ai = find_unmatched_o_paren (and_sen, ai);
+    {
+      ai = find_unmatched_o_paren (and_sen, ai);
+      if (ai == -1)
+	return _("There must be connectives on both sentences.");
+    }
 
   int ret_chk;
   ret_chk = sexpr_find_unmatched (and_sen, con_sen, &ai, &ci);
@@ -1491,9 +1482,13 @@ proc_ep (unsigned char * prem, unsigned char * conc)
     return NULL;
 
   ai--;
+  if (ai < 0)
+    return _("Exportation constructed incorrectly.");
 
   ai = find_unmatched_o_paren (and_sen, ai);
-
+  if (ai == -1)
+    return _("Exportation constructed incorrectly.");
+  
   int tmp_pos;
   unsigned char * tmp_str;
 
@@ -1504,6 +1499,7 @@ proc_ep (unsigned char * prem, unsigned char * conc)
   int ftc;
   unsigned char * lsen, * rsen;
 
+  lsen = rsen = NULL;
   ftc = sexpr_find_top_connective (tmp_str, S_CON, &lsen, &rsen);
   if (ftc == -1)
     return NULL;
@@ -1517,51 +1513,53 @@ proc_ep (unsigned char * prem, unsigned char * conc)
       return _("There must be a conditional in both sentences.");
     }
 
-  unsigned char * a_lsen, * a_rsen;
+  vec_t * a_sens;
 
-  ftc = sexpr_find_top_connective (lsen, S_AND, &a_lsen, &a_rsen);
+  a_sens = init_vec (sizeof (char *));
+  if (!a_sens)
+    return NULL;
+
+  ftc = sexpr_get_generalities (lsen, S_AND, a_sens);
   if (ftc == -1)
     return NULL;
   free (lsen);
 
-  if (ftc < 0)
+  if (ftc == 1)
     {
+      destroy_str_vec (a_sens);
       free (rsen);
-      if (a_lsen)  free (a_lsen);
-      if (a_rsen)  free (a_rsen);
-
-      return _("There must be a conjunction in the conjunction sentence.");
+      return _("There must be a conjunctions in the conjunction sentence.");
     }
 
   unsigned char * oth_sen;
-  int oth_pos;
-
-  oth_sen = (unsigned char *) calloc (a_len + 1, sizeof (char));
-  if (!oth_sen)
-    {
-      perror (NULL);
-      return NULL;
-    }
+  int oth_pos, alloc_size;
 
+  alloc_size = a_len + (3 + S_CL) * a_sens->num_stuff;
+  oth_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+  CHECK_ALLOC (oth_sen, NULL);
   strncpy (oth_sen, and_sen, ai);
   oth_pos = ai;
 
-  oth_pos += sprintf (oth_sen + oth_pos, "%s %s (%s %s %s)",
-		      S_CON, a_lsen, S_CON, a_rsen, rsen);
+  for (i = 0; i < a_sens->num_stuff; i++)
+    {
+      oth_pos += sprintf (oth_sen + oth_pos, "(%s %s ",
+			  S_CON, vec_str_nth (a_sens, i));
+    }
 
-  free (a_lsen);
-  free (a_rsen);
+  oth_pos += sprintf (oth_sen + oth_pos, "%s", rsen);
+  for (i = 0; i < a_sens->num_stuff; i++)
+    oth_pos += sprintf (oth_sen + oth_pos, ")");
+
+  destroy_str_vec (a_sens);
   free (rsen);
 
-  strcpy (oth_sen + oth_pos, and_sen + tmp_pos);
+  strcpy (oth_sen + oth_pos, and_sen + tmp_pos + 1);
 
   char * ret_str;
-
   if (and_sen == conc)
     ret_str = proc_ep (con_sen, oth_sen);
   else
     ret_str = proc_ep (oth_sen, con_sen);
-
   if (!ret_str)
     return NULL;
   free (oth_sen);
@@ -1605,7 +1603,21 @@ proc_sb (unsigned char * prem, unsigned char * conc)
 
   si = li = i;
   if (sh_sen[si] == '(')
-    si--;
+    {
+      si--;
+      if (si < 0)
+	return _("Subsumption constructed incorrectly.");
+    }
+  else if (si > 0
+	   && (!strncmp (sh_sen + si - 1, S_AND, S_CL)
+	       || !strncmp (sh_sen + si - 1, S_OR, S_CL)
+	       || !strncmp (sh_sen + si - 1, S_CON, S_CL)
+	       || !strncmp (sh_sen + si - 1, S_BIC, S_CL)))
+    {
+      while (sh_sen[si] != '(')
+	si--;
+      si--;
+    }
 
   si = find_unmatched_o_paren (sh_sen, si);
   if (si == -1)
@@ -1625,11 +1637,13 @@ proc_sb (unsigned char * prem, unsigned char * conc)
       if (ret_chk == -1)
 	return NULL;
 
-      //TODO: Figure out a better error message.
       if (li < 0)
-	return _("ERROR.");
+	return _("Subsumption constructed incorrectly.");
     }
 
+  if (ln_sen[li] != '(')
+    return _("There must be connectives on both sentences.");
+
   int tmp_pos;
   unsigned char * tmp_str;
 
@@ -1643,6 +1657,7 @@ proc_sb (unsigned char * prem, unsigned char * conc)
   unsigned char * t_lsen, * t_rsen, tconn[S_CL + 1];
 
   tconn[0] = '\0';
+  t_lsen = t_rsen = NULL;
   ftc = sexpr_find_top_connective (tmp_str, tconn, &t_lsen, &t_rsen);
   if (ftc == -1)
     return NULL;
@@ -1670,6 +1685,7 @@ proc_sb (unsigned char * prem, unsigned char * conc)
 
   r_len = strlen (t_rsen);
   conn[0] = '\0';
+  lsen = rsen = NULL;
   ftc = sexpr_find_top_connective (t_rsen, conn, &lsen, &rsen);
   if (ftc == -1)
     return NULL;
@@ -1706,12 +1722,7 @@ proc_sb (unsigned char * prem, unsigned char * conc)
   int oth_pos;
 
   oth_sen = (unsigned char *) calloc (l_len - r_len - S_CL - 3, sizeof (char));
-  if (!oth_sen)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
+  CHECK_ALLOC (oth_sen, NULL);
   strncpy (oth_sen, ln_sen, li);
   oth_pos = li;
 
