@@ -22,6 +22,12 @@
 #include "var.h"
 #include "list.h"
 
+/* Checks for a negation on a sexpr string.
+ *  input:
+ *    in_str - the sexpr text on which to check for a negation.
+ *  output:
+ *    1 if there is a negation, 0 otherwise.
+ */
 int
 sexpr_not_check (unsigned char * in_str)
 {
@@ -36,23 +42,30 @@ sexpr_not_check (unsigned char * in_str)
   return 1;
 }
 
+/* Adds a negation to a sexpr string.
+ *  input:
+ *    in_str - the sexpr text to which to add a negation.
+ *  output:
+ *    The modified string, or NULL on memory error.
+ */
 unsigned char *
 sexpr_add_not (unsigned char * in_str)
 {
   unsigned char * not_in_str;
 
   not_in_str = (unsigned char *) calloc (strlen (in_str) + S_NL + 3, sizeof (char));
-  if (!not_in_str)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
+  CHECK_ALLOC (not_in_str, NULL);
   sprintf (not_in_str, "(%s %s)\0", S_NOT, in_str);
 
   return not_in_str;
 }
 
+/* Eliminates a negation from a sexpr string.
+ *  input:
+ *    in_str - the sexpr text from which to remove a negation.
+ *  output:
+ *    The modified string, or NULL on memory error.
+ */
 unsigned char *
 sexpr_elim_not (unsigned char * in_str)
 {
@@ -61,18 +74,21 @@ sexpr_elim_not (unsigned char * in_str)
 
   in_len = strlen (in_str);
   out_str = (unsigned char *) calloc (in_len - S_NL - 2, sizeof (char));
-  if (!out_str)
-    {
-      perror (NULL);
-      return NULL;
-    }
-
+  CHECK_ALLOC (out_str, NULL);
   strncpy (out_str, in_str + 2 + S_NL, in_len - (3 + S_NL));
   out_str[in_len - (3 + S_NL)] = '\0';
 
   return out_str;
 }
 
+/* Gets the generalities from a sexpr string.
+ *  input:
+ *    in_str - the sexpr text from which to obtain the generalities.
+ *    conn - the connective to check for, or an empty string to check for any.
+ *    vec - the string vector to hold the generalities.
+ *  output:
+ *    The size of vec, or -1 on memory error.
+ */
 int
 sexpr_get_generalities (unsigned char * in_str, unsigned char * conn, vec_t * vec)
 {
@@ -137,15 +153,12 @@ sexpr_get_generalities (unsigned char * in_str, unsigned char * conn, vec_t * ve
 	  tmp_pos = pos;
 	  while (in_str[tmp_pos] != ' ' && in_str[tmp_pos] != ')')
 	    tmp_pos++;
-	  tmp_str = (unsigned char *) calloc (tmp_pos - pos + 1, sizeof (char));
-	  if (!tmp_str)
-	    {
-	      perror (NULL);
-	      return -1;
-	    }
 
+	  tmp_str = (unsigned char *) calloc (tmp_pos - pos + 1, sizeof (char));
+	  CHECK_ALLOC (tmp_str, -1);
 	  strncpy (tmp_str, in_str + pos, tmp_pos - pos);
 	  tmp_str[tmp_pos - pos] = '\0';
+
 	  ret_chk = vec_str_add_obj (vec, tmp_str);
 	  if (ret_chk < 0)
 	    return -1;
@@ -158,6 +171,15 @@ sexpr_get_generalities (unsigned char * in_str, unsigned char * conn, vec_t * ve
   return vec->num_stuff;
 }
 
+/* Gets the top connectives from a sexpr string.
+ *  input:
+ *    in_str - the sexpr text from which to get the top connective.
+ *    conn - the connective to check for, or an empty string to check for any.
+ *    lsen - a pointer to a string to hold the left sentence.
+ *    rsen - a pointer to a string to hold the right sentence.
+ *  output:
+ *    0 on success, -1 on memory error, -2 if there aren't two generalities.
+ */
 int
 sexpr_find_top_connective (unsigned char * in_str, unsigned char * conn,
 			   unsigned char ** lsen, unsigned char ** rsen)
@@ -189,20 +211,11 @@ sexpr_find_top_connective (unsigned char * in_str, unsigned char * conn,
   g1_len = strlen (gen_1);
 
   *lsen = (unsigned char *) calloc (g0_len + 1, sizeof (char));
-  if (!(*lsen))
-    {
-      perror (NULL);
-      return -3;
-    }
+  CHECK_ALLOC (*lsen, -1);
+  strcpy (*lsen, gen_0);
 
   *rsen = (unsigned char *) calloc (g1_len + 1, sizeof (char));
-  if (!(*rsen))
-    {
-      perror (NULL);
-      return -3;
-    }
-
-  strcpy (*lsen, gen_0);
+  CHECK_ALLOC (*rsen, -1);
   strcpy (*rsen, gen_1);
 
   destroy_str_vec (vec);
@@ -210,6 +223,13 @@ sexpr_find_top_connective (unsigned char * in_str, unsigned char * conn,
   return 0;
 }
 
+/* Finds an unmatched opening parenthesis.
+ *  input:
+ *    in_str - the string on which to check for an unmatched opening parenthesis.
+ *    in_pos - the initial position.
+ *  output:
+ *    The position of the unmatched opening parenthesis, or -1 on memory error.
+ */
 int
 find_unmatched_o_paren (unsigned char * in_str, int in_pos)
 {
@@ -234,6 +254,14 @@ find_unmatched_o_paren (unsigned char * in_str, int in_pos)
   return pos;
 }
 
+/* Find unmatched sentence parts from an opening parenthesis.
+ *  input:
+ *    sen_a, sen_b - the sentences to check.
+ *    ai, bi - integer pointers that receive the positions.
+ *      These both must start on an opening parenthesis.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
 int
 sexpr_find_unmatched (unsigned char * sen_a, unsigned char * sen_b,
 		      int * ai, int * bi)
@@ -277,6 +305,16 @@ sexpr_find_unmatched (unsigned char * sen_a, unsigned char * sen_b,
   return 0;
 }
 
+/* Get the predicate arguments from a sexpr string.
+ *  input:
+ *    in_str - the sexpr text from which to get the predicate arguments.
+ *    pred - a string pointer that receives the predicate symbol.
+ *    vec - a string vector that receives the arguments.
+ *  output:
+ *    The number of arguments on success,
+ *    0 on error,
+ *    -1 on memory error.
+ */
 int
 sexpr_get_pred_args (unsigned char * in_str, unsigned char ** pred, vec_t * vec)
 {
@@ -294,11 +332,7 @@ sexpr_get_pred_args (unsigned char * in_str, unsigned char ** pred, vec_t * vec)
   if (pred)
     {
       *pred = (unsigned char *) calloc (oth_pos - pos + 1, sizeof (char));
-      if (!(*pred))
-	{
-	  perror (NULL);
-	  return -1;
-	}
+      CHECK_ALLOC (*pred, -1);
 
       strncpy (*pred, in_str + pos, oth_pos - pos);
       (*pred)[oth_pos - pos] = '\0';
@@ -339,15 +373,12 @@ sexpr_get_pred_args (unsigned char * in_str, unsigned char ** pred, vec_t * vec)
 	  tmp_pos = pos;
 	  while (elm_str[tmp_pos] != ' ' && elm_str[tmp_pos] != '\0')
 	    tmp_pos++;
-	  tmp_str = (unsigned char *) calloc (tmp_pos - pos + 1, sizeof (char));
-	  if (!tmp_str)
-	    {
-	      perror (NULL);
-	      return -1;
-	    }
 
+	  tmp_str = (unsigned char *) calloc (tmp_pos - pos + 1, sizeof (char));
+	  CHECK_ALLOC (tmp_str, -1);
 	  strncpy (tmp_str, elm_str + pos, tmp_pos - pos);
 	  tmp_str[tmp_pos - pos] = '\0';
+
 	  ret_chk = vec_str_add_obj (vec, tmp_str);
 	  if (ret_chk < 0)
 	    return -1;
@@ -360,6 +391,14 @@ sexpr_get_pred_args (unsigned char * in_str, unsigned char ** pred, vec_t * vec)
   return vec->num_stuff;
 }
 
+/* Eliminates a quantifier from a sexpr string.
+ *  input:
+ *    in_str - the quantifier from which to eliminate the quantifier.
+ *    quant - receives the quantifier.
+ *    var - a string pointer that receives the variable.
+ *  output:
+ *    the scope of the quantifier, or NULL on error.
+ */
 unsigned char *
 sexpr_elim_quant (unsigned char * in_str, unsigned char * quant,
 		  unsigned char ** var)
@@ -384,11 +423,7 @@ sexpr_elim_quant (unsigned char * in_str, unsigned char * quant,
   quant[S_CL] = '\0';
 
   *var = (unsigned char *) calloc (tmp_pos - (2 + S_CL), sizeof (char));
-  if (!(*var))
-    {
-      perror (NULL);
-      return NULL;
-    }
+  CHECK_ALLOC (*var, NULL);
   strncpy (*var, in_str + 3 + S_CL, tmp_pos - (3 + S_CL));
 
   if (in_str[tmp_pos + 2] != '(')
@@ -401,6 +436,13 @@ sexpr_elim_quant (unsigned char * in_str, unsigned char * quant,
   return tmp_str;
 }
 
+/* Get the offsets from the start of a quantifier's scope of its variable.
+ *  input:
+ *    in_str - the sexpr string from which to obtain the offsets.
+ *    var - a vector of integers that receives the offsets.
+ *  output:
+ *    The size of vars on success, -1 on memory error, -2 on general error.
+ */
 int
 sexpr_get_quant_vars (unsigned char * in_str, vec_t * vars)
 {
@@ -436,6 +478,16 @@ sexpr_get_quant_vars (unsigned char * in_str, vec_t * vars)
   return vars->num_stuff;
 }
 
+/* Replaces a variable in a sexpr string.
+ *  input:
+ *    in_str - the sexpr text to replace the variables of.
+ *    new_var - the new variable.
+ *    old_var - the old variable.
+ *    off_var - the variable offsets in in_str.
+ *    out_str - the modified string.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
 int
 sexpr_replace_var (unsigned char * in_str, unsigned char * new_var,
 		   unsigned char * old_var, vec_t * off_var,
@@ -445,11 +497,7 @@ sexpr_replace_var (unsigned char * in_str, unsigned char * new_var,
 
   old_len = strlen (old_var);
   *out_str = (unsigned char *) calloc (strlen (in_str) + strlen (new_var) * off_var->num_stuff, sizeof (char));
-  if (!(*out_str))
-    {
-      perror (NULL);
-      return -1;
-    }
+  CHECK_ALLOC (*out_str, -1);
 
   i = 0;
   cur_off = vec_nth (off_var, i);
@@ -473,6 +521,20 @@ sexpr_replace_var (unsigned char * in_str, unsigned char * new_var,
   return 0;
 }
 
+/* Processes the standard quantifier inference rules.
+ *  input:
+ *    quant_sen - the quantifier sentence.
+ *    elim_sen - the other sentence.
+ *    quant - the quantifier.
+ *    cons - the constraints - 0 normally, 1 for ug, 2 for ei.
+ *    cur_vars - the current variables, or NULL if they're not neccessary.
+ *  output:
+ *    0 - success
+ *    1 - the strings are the same
+ *    -1 - memory error
+ *    -2 - general error
+ *    -3 - variable error
+ */
 int
 sexpr_quant_infer (unsigned char * quant_sen, unsigned char * elim_sen,
 		   unsigned char * quant, int cons, vec_t * cur_vars)
@@ -569,11 +631,7 @@ sexpr_quant_infer (unsigned char * quant_sen, unsigned char * elim_sen,
 	q_pos++;
 
       new_var = (unsigned char *) calloc (q_pos - e_pos + 1, sizeof (char));
-      if (!new_var)
-	{
-	  perror (NULL);
-	  return -1;
-	}
+      CHECK_ALLOC (new_var, -1);
 
       strncpy (new_var, elim_sen + e_pos, q_pos - e_pos);
       new_var[q_pos - e_pos] = '\0';
@@ -649,6 +707,14 @@ sexpr_quant_infer (unsigned char * quant_sen, unsigned char * elim_sen,
   return 0;
 }
 
+/* Determines the positions in a string of a variable.
+ *  input:
+ *    in_str - the sexpr text of which to determine the positions.
+ *    var - the variable.
+ *    offsets - an integer vector that stores the offsets.
+ *  output:
+ *    The size of offsets on success, -1 on memory error.
+ */
 int
 sexpr_find_vars (unsigned char * in_str, unsigned char * var, vec_t * offsets)
 {
@@ -675,9 +741,18 @@ sexpr_find_vars (unsigned char * in_str, unsigned char * var, vec_t * offsets)
   return offsets->num_stuff;
 }
 
+/* Collects variables from a sexpr string.
+ *  input:
+ *    in_str - the sexpr text from which to collect variables.
+ *    vars - a string vector that holds the variables.
+ *    quant - whether or not quantifier variables are being looked for.
+ *  output:
+ *    The size of vars on success, -1 on memory error.
+ */
 int
 sexpr_parse_vars (unsigned char * in_str, vec_t * vars, int quant)
 {
+  // There are no variables in this string.
   if (in_str[0] != '(')
     return 0;
 
@@ -709,12 +784,7 @@ sexpr_parse_vars (unsigned char * in_str, vec_t * vars, int quant)
 	pos++;
 
       new_var = (unsigned char *) calloc (pos - i + 1, sizeof (char));
-      if (!new_var)
-	{
-	  perror (NULL);
-	  return -1;
-	}
-
+      CHECK_ALLOC (new_var, -1);
       strncpy (new_var, in_str + i, pos - i);
       new_var[pos - i] = '\0';
       i = pos;
@@ -736,6 +806,14 @@ sexpr_parse_vars (unsigned char * in_str, vec_t * vars, int quant)
   return vars->num_stuff;
 }
 
+/* Collect variables from a sentence to a list.
+ *  input:
+ *    vars - the list of variables from a proof.
+ *    text - the text of the sentence.
+ *    arb - whether or not the variables are arbitrary.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
 int
 sexpr_collect_vars_to_proof (list_t * vars, unsigned char * text, int arb)
 {
@@ -785,15 +863,24 @@ sexpr_collect_vars_to_proof (list_t * vars, unsigned char * text, int arb)
 	  if (!var)
 	    return -1;
 
-	  itm = ls_ins_obj (vars, var, vars->tail);
+	  itm = ls_push_obj (vars, var);
 	  if (!itm)
 	    return -1;
 	}
     }
 
   destroy_str_vec (sen_vars);
+  return 0;
 }
 
+/* Collect the object ids from a sexpr sentence.
+ *  input:
+ *    sen - the sentence text.
+ *    ids - a ponter to an array that receives the ids.
+ *    sen_ids - ids for sentence parts from previous sentences.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
 int
 sexpr_get_ids (unsigned char * sen, int ** ids, vec_t * sen_ids)
 {
@@ -816,11 +903,7 @@ sexpr_get_ids (unsigned char * sen, int ** ids, vec_t * sen_ids)
     }
 
   *ids = (int *) calloc (sen_len, sizeof (int));
-  if (!(*ids))
-    {
-      perror (NULL);
-      return -1;
-    }
+  CHECK_ALLOC (*ids, -1);
 
   j = 0;
   for (i = 0; i < sen_len; i++)
@@ -898,6 +981,26 @@ sexpr_get_ids (unsigned char * sen, int ** ids, vec_t * sen_ids)
 	  continue;
 	}
 
+      if (sen[i] == '<')
+	{
+	  (*ids)[j++] = SEN_ID_LT;
+	  continue;
+	}
+
+      if (!strncmp (sen + i, S_ELM, S_CL))
+	{
+	  (*ids)[j++] = SEN_ID_ELM;
+	  i += (S_CL - 1);
+	  continue;
+	}
+
+      if (!strncmp (sen + i, S_NIL, S_CL))
+	{
+	  (*ids)[j++] = SEN_ID_NIL;
+	  i += (S_CL - 1);
+	  continue;
+	}
+
       if (isalnum (sen[i]))
 	{
 	  unsigned char * new_sen;
@@ -934,12 +1037,7 @@ sexpr_get_ids (unsigned char * sen, int ** ids, vec_t * sen_ids)
 
 	      new_sen_id.sen = (unsigned char *) calloc (end - start + 1,
 							 sizeof (char));
-	      if (!new_sen_id.sen)
-		{
-		  perror (NULL);
-		  return -1;
-		}
-
+	      CHECK_ALLOC (new_sen_id.sen, -1);
 	      strncpy (new_sen_id.sen, sen + start, end - start);
 	      new_sen_id.sen[end - start] = '\0';
 
